/* vim: set noexpandtab shiftwidth=8 cino=:
 ***************************************************************************
 *            controls.cpp
 *
 *  Sat Jun  4 08:18:38 2005
 *  Copyright  2005  Joe Venzon
 *  joe@venzon.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "controls.h"
#include "singleton.h"
#include "game.h"
extern GAME game;

void CONTROL::Dump()
{
	cout << "Name: " << controlname << endl;
	cout << "Type: " << (int) controltype << " = ";
	if (controltype == Joy)
	{
		cout << "Joy" << endl;
		cout << "Joy number: " << joynum << endl;
		cout << "Input type: ";
		if (joytype == Button)
		{
			cout << "Button" << endl;
			cout << "Button number: " << joybutton << endl;
			cout << "One time: " << onetime << endl;
		}
		else if (joytype == Axis)
		{
			cout << "Axis" << endl;
			cout << "Axis number: " << joyaxis << endl;
			cout << "Axis type: " << joyaxistype << endl;
		}
		else
			cout << "Unknown" << endl;
	}
	else if (controltype == Key)
	{
		cout << "Key" << endl;
		cout << "Keycode: " << keycode << endl;
		cout << "One time: " << onetime << endl;
	}
	else if( controltype == Mouse )
	{
		cout << "Mouse" << endl;
		if( mousetype == MButton )
		{
			cout << "Button: " << mbutton << endl;
			cout << "One time: " << onetime << endl;
		}
		else if( mousetype == Motion )
		{
			cout << "Direction: " << mdir << endl;
		}
		else
		{
			cout << "unknown" << endl;
		}
	}
	else
		cout << "Unknown" << endl;
}

void GAMECONTROLS::LoadControls()
{
	string controls_configfile = game.settings.GetSettingsDir() + "/controls.config";
	string controls_oldfile = game.settings.GetSettingsDir() + "/controls";
	if( game.utility.FileExists( controls_configfile ) )
	{
		LoadControlsConfigFile( controls_configfile );
	}
	else
	{
		LoadControlsOldFormat( controls_oldfile );
	}
}

void GAMECONTROLS::LoadControlsConfigFile(string fname)
{
	list<string> sections;

	UpdateSettings();
	LoadCalibration();

	CONFIGFILE controls_config( fname );
	controls_config.GetSectionList( sections );

	controls.clear();
	
	log << "Loading controls" << endl;
	
	for( list<string>::const_iterator section = sections.begin(); section != sections.end(); ++section )
	{
		list<string> params;
		controls_config.GetParamList( params, *section );
		CONTROL newctrl;
		string type = "";
		string name = "";
		controls_config.GetParam( *section + ".type", type );
		controls_config.GetParam( *section + ".name", name );
		newctrl.SetType( type );
		newctrl.SetName( name );
		if( newctrl.GetType() == Joy )
		{
			int joy_num = 0;
			string joy_type = "";
			controls_config.GetParam( *section + ".joy_index", joy_num );
			controls_config.GetParam( *section + ".joy_type", joy_type );
			newctrl.SetJoyNum( joy_num );
			newctrl.SetJoyType( joy_type );
			if( newctrl.GetJoyType() == Button )
			{
				int joy_btn = 0;
				bool joy_btn_down = false;
				bool joy_btn_once = false;
				controls_config.GetParam( *section + ".joy_button", joy_btn );
				controls_config.GetParam( *section + ".down", joy_btn_down );
				controls_config.GetParam( *section + ".once", joy_btn_once );
				newctrl.SetJoyButton( joy_btn );
				newctrl.SetJoyPushDown( joy_btn_down );
				newctrl.SetOneTime( joy_btn_once );
			}
			else if( newctrl.GetJoyType() == Axis )
			{
				int joy_axis = 0;
				string axis_type = "";
				float deadzone = 0.0;
				float exponent = 0.0;
				float gain = 0.0;
				controls_config.GetParam( *section + ".joy_axis", joy_axis );
				controls_config.GetParam( *section + ".joy_axis_type", axis_type );
				controls_config.GetParam( *section + ".deadzone", deadzone );
				controls_config.GetParam( *section + ".exponent", exponent );
				controls_config.GetParam( *section + ".gain", gain );
				newctrl.SetJoyAxis( joy_axis );
				if( axis_type == "positive" )
				{
					newctrl.SetJoyAxisType( Positive );
				}
				else if( axis_type == "negative" )
				{
					newctrl.SetJoyAxisType( Negative );
				}
				else if( axis_type == "both" )
				{
					newctrl.SetJoyAxisType( Both );
				}
				else
				{
					log << "Error parsing controls, invalid joystick axis type" << endl;
					continue;
				}
				newctrl.SetDeadzone( deadzone );
				newctrl.SetExponent( exponent );
				newctrl.SetGain( gain );
			}
			else
			{
				log << "Error parsing controls, invalid joystick type" << endl;
				continue;
			}
		}
		else if( newctrl.GetType() == Key )
		{
			string key = "";
			bool key_down = false;
			bool key_once = false;
			controls_config.GetParam( *section + ".key", key );
			controls_config.GetParam( *section + ".down", key_down );
			controls_config.GetParam( *section + ".once", key_once );
			newctrl.SetKeyCode( game.keyman.GetKey( key ) );
			newctrl.SetKeyPushDown( key_down );
			newctrl.SetOneTime( key_once );
		}
		else if( newctrl.GetType() == Mouse )
		{
			string mouse_type = "";
			string mouse_direction = "";
			int mouse_btn = 0;
			bool mouse_btn_down = false;
			bool mouse_btn_once = false;
			controls_config.GetParam( *section + ".mouse_type", mouse_type );
			newctrl.SetMouseType( mouse_type );
			if( newctrl.GetMouseType() == MButton )
			{
				controls_config.GetParam( *section + ".mouse_button", mouse_btn );
				controls_config.GetParam( *section + ".down", mouse_btn_down );
				controls_config.GetParam( *section + ".once", mouse_btn_once );
				newctrl.SetMouseButton( mouse_btn );
				newctrl.SetMousePushDown( mouse_btn_down );
				newctrl.SetOneTime( mouse_btn_once );
			}
			else if( newctrl.GetMouseType() == Motion )
			{
				controls_config.GetParam( *section + ".mouse_motion", mouse_direction );
				newctrl.SetMouseDirection( mouse_direction );
			}
			else
			{
				log << "Error parsing controls, invalid mouse type" << endl;
				continue;
			}
		}
		else
		{
			log << "Error parsing controls, invalid control type" << endl;
			continue;
		}
		
		controls.push_back(newctrl);
	}

/*
	//for debugging
	for( list<CONTROL>::iterator i = controls.begin(); i != controls.end(); ++i )
	{
		i->Dump();
		cout << endl;
	}
*/

}

void GAMECONTROLS::LoadControlsOldFormat(string fname)
{
	UpdateSettings();

	
	LoadCalibration();
	
	ifstream controlfile(fname.c_str());
	
	if (!controlfile)
	{
		cerr << "Couldn't open control file: " << fname << endl;
		Clear();
		return;
	}
	
	//first pass to get the number of lines
	string input;
	input = game.utility.sGetLine(controlfile);
	int numcontrols = 0;
	while (input != "!!!END OF FILE!!!")
	{
		input = game.utility.sGetLine(controlfile);
		if(input.find("joysticktype") == string::npos && input.find("!!!END OF FILE!!!") == string::npos)
			numcontrols++;
	}
	
	//delete [] controls;
	controls.clear();
	
	log << "Loading controls" << endl;
	//controls = new CONTROL [numcontrols];
	
	controlfile.close();
	controlfile.open(fname.c_str());
	controlfile.clear();
	
	//second pass to actually read the file
	
	int i;
	for (i = 0; i < numcontrols; i++)
	{
		input = game.utility.sGetParam(controlfile);
		if (input == "joysticktype")
		{

			joysticktype = game.utility.sGetParam(controlfile);
			//input = game.utility.sGetParam(controlfile);
			compensation = game.utility.sGetParam(controlfile);
			if (joysticktype != "wheel" && joysticktype != "joystick")
			{
				cout << "invalid joystick type in " + game.settings.GetSettingsDir() + "/controls: " << joysticktype << endl;
				joysticktype = "joystick";
			}
			if (compensation != "off" && compensation != "low" && compensation != "med" && compensation != "high" && compensation != "900to200" && compensation != "speed1" && compensation != "speed2")
			{
				cout << "invalid joystick touchiness compensation in " + game.settings.GetSettingsDir() + "/controls: " << compensation << endl;
				compensation = "off";
			}
			i--;
		}
		else
		{
			CONTROL newctrl;
			newctrl.SetName(input);
			//controls[i].SetName(game.utility.sGetParam(controlfile));
			newctrl.SetType(game.utility.sGetParam(controlfile));
			if (newctrl.GetType() == Joy)
			{
				newctrl.SetJoyNum(game.utility.iGetParam(controlfile));
				newctrl.SetJoyType(game.utility.sGetParam(controlfile));
				if (newctrl.GetJoyType() == Button)
				{
					newctrl.SetJoyButton(game.utility.iGetParam(controlfile));
					newctrl.SetOneTime(game.utility.bGetParam(controlfile));
					
					//if (controls[i].GetOneTime())
						newctrl.SetJoyPushDown(game.utility.bGetParam(controlfile));
				}
				else if (newctrl.GetJoyType() == Axis)
				{
					newctrl.SetJoyAxis(game.utility.iGetParam(controlfile));
					string axis_type = game.utility.sGetParam(controlfile);
					if (axis_type == "positive")
						newctrl.SetJoyAxisType(Positive);
					else if (axis_type == "negative")
						newctrl.SetJoyAxisType(Negative);
					else if (axis_type == "both")
						newctrl.SetJoyAxisType(Both);
					else
					{
						ControlParseError(i, controlfile);
						i = numcontrols; //exit loop
					}
					float ftemp = game.utility.fGetParam(controlfile);
					newctrl.SetDeadzone(ftemp);
					ftemp = game.utility.fGetParam(controlfile);
					newctrl.SetExponent(ftemp);
					ftemp = game.utility.fGetParam(controlfile);
					newctrl.SetGain(ftemp);
				}
				else
				{
					ControlParseError(i, controlfile);
					i = numcontrols; //exit loop
				}
			}
			else if (newctrl.GetType() == Key)
			{
				newctrl.SetKeyCode(game.keyman.GetKey(game.utility.sGetParam(controlfile)));
				newctrl.SetOneTime(game.utility.bGetParam(controlfile));
				newctrl.SetKeyPushDown(game.utility.bGetParam(controlfile));
			}
			else if( newctrl.GetType() == Mouse )
			{
				newctrl.SetMouseType( game.utility.sGetParam( controlfile ) );
				if( newctrl.GetMouseType() == MButton )
				{
					newctrl.SetMouseButton( game.utility.sGetParam( controlfile ) );
					newctrl.SetOneTime( game.utility.bGetParam( controlfile ) );
					newctrl.SetMousePushDown( game.utility.bGetParam( controlfile ) );
				}
				else if( newctrl.GetMouseType() == Motion )
				{
					newctrl.SetMouseDirection( game.utility.sGetParam( controlfile ) );
				}
				else
				{
					ControlParseError( i, controlfile );
					i = numcontrols; // exit loop
				}
			}
			else
			{
				ControlParseError(i, controlfile);
				i = numcontrols; //exit loop
			}
			
			controls.push_back(newctrl);
		}
	}

/*
	//for debugging
	for( list<CONTROL>::iterator i = controls.begin(); i != controls.end(); ++i )
	{
		i->Dump();
		cout << endl;
	}
*/
}

void GAMECONTROLS::ControlParseError(int line, ifstream & ffrom)
{
	log << "Error parsing control number " << line << endl;
	ffrom.unget();
	ffrom.unget();
	ffrom.unget();
	ffrom.unget();
	ffrom.unget();
	string errorword;
	ffrom >> errorword;
	ffrom.seekg(0, ios_base::beg);
	string errorline;
	int i;
	for (i = 0; i <= line; i++)
		errorline = game.utility.sGetLine(ffrom);
	log << "The line was: " << errorline << endl;
	log << "The error is in the vicinity of: " << errorword << endl;
}

GAMECONTROLS::GAMECONTROLS() : log(VDrift::LOGGING::GetLogger("controls"))
{
	log.SetLevel(LOG_INFO);
	log.AddLog(new ofstream((game.settings.GetSettingsDir() + "/logs/controls.log").c_str()), LOG_ALL);
	Clear();
}

void GAMECONTROLS::Clear()
{
	controls.clear();
	//numcontrols = 0;
	
	joysticktype = "joystick";
	compensation = "off";
	
	int i;
	for (i = 0; i < MAX_JOYSTICKS; i++)
	{
		int m;
		for (m = 0; m < MAX_AXES; m++)
		{
			calibration[i][m][0] = -1;
			calibration[i][m][1] = 1;
		}
	}
}

extern bool verbose_output;
GAMECONTROLS::~GAMECONTROLS()
{
	//delete [] controls;
	controls.clear();
	//log.close();
}

/*list <CONTROL> & GAMECONTROLS::GetControls()
{
	return controls;
}*/

int GAMECONTROLS::GetNumControls()
{
	//return numcontrols;
	return controls.size();
}

void GAMECONTROLS::SetControls(CONTROL * newcontrols, int newnum)
{
	/*delete [] controls;
	controls = newcontrols;
	numcontrols = newnum;*/
	
	controls.clear();
	for (int i = 0; i < newnum; i++)
		controls.push_back(newcontrols[i]);
}

void GAMECONTROLS::WriteControlFile()
{
	WriteControlFile(game.settings.GetSettingsDir() + "/controls.config");
}

void GAMECONTROLS::WriteControlFile(string fname)
{
	CONFIGFILE controls_config;
	
	log << "Writing controls file " << fname << endl;
	
	// save general control settings in global config file
	game.settings.Set( "joystick.type", joysticktype );
	game.settings.Set( "joystick.touchcomp", compensation );

	int i = 0;
	for (ControlIteratorReset(); ControlIteratorGetControl() != NULL; ControlIteratorIncrement())
	{
		std::ostringstream ss;
		ss << "control mapping ";
		ss << std::setfill('0') << std::setw(2) << i;
		string ctrl_id = ss.str();
		CONTROL & curctrl = *(ControlIteratorGetControl());
		string name = curctrl.GetName();
		controls_config.SetParam( ctrl_id + ".name", name );
		if (curctrl.GetType() == Joy)
		{
			controls_config.SetParam( ctrl_id + ".type", string("joy") );
			controls_config.SetParam( ctrl_id + ".joy_index", curctrl.GetJoyNum() );
			
			if (curctrl.GetJoyType() == Axis)
			{
				controls_config.SetParam( ctrl_id + ".joy_type", string("axis") );
				controls_config.SetParam( ctrl_id + ".joy_axis", curctrl.GetJoyAxis() );
				switch (curctrl.GetJoyAxisType()) {
					case Positive:
						controls_config.SetParam( ctrl_id + ".joy_axis_type", string("positive") );
						break;
					case Negative:
						controls_config.SetParam( ctrl_id + ".joy_axis_type", string("negative") );
						break;
					case Both:
						controls_config.SetParam( ctrl_id + ".joy_axis_type", string("both") );
						break;
				}
				controls_config.SetParam( ctrl_id + ".deadzone", curctrl.GetDeadzone() );
				controls_config.SetParam( ctrl_id + ".exponent", curctrl.GetExponent() );
				controls_config.SetParam( ctrl_id + ".gain", curctrl.GetGain() );
			}
			else if (curctrl.GetJoyType() == Button)
			{
				controls_config.SetParam( ctrl_id + ".joy_type", string("button") );
				controls_config.SetParam( ctrl_id + ".joy_button", curctrl.GetJoyButton() );
				controls_config.SetParam( ctrl_id + ".once", curctrl.GetOneTime() );
				controls_config.SetParam( ctrl_id + ".down", curctrl.GetJoyPushDown() );
			}
		}
		else if (curctrl.GetType() == Key)
		{
			controls_config.SetParam( ctrl_id + ".type", string("key") );
			controls_config.SetParam( ctrl_id + ".key", game.keyman.GetKeyName(curctrl.GetKeyCode()) );
			controls_config.SetParam( ctrl_id + ".once", curctrl.GetOneTime() );
			controls_config.SetParam( ctrl_id + ".down", curctrl.GetKeyPushDown() );
		}
		else if( curctrl.GetType() == Mouse )
		{
			controls_config.SetParam( ctrl_id + ".type", string("mouse") );
			if( curctrl.GetMouseType() == MButton )
			{
				controls_config.SetParam( ctrl_id + ".mouse_type", string("button") );
				controls_config.SetParam( ctrl_id + ".mouse_button", curctrl.GetMouseButton() );
				controls_config.SetParam( ctrl_id + ".once", curctrl.GetOneTime() );
				controls_config.SetParam( ctrl_id + ".down", curctrl.GetMousePushDown() );
			}
			else if( curctrl.GetMouseType() == Motion )
			{
				string direction;
				int mdir = curctrl.GetMouseDirection();
				if( mdir == Up )
				{
					direction = "up";
				}
				else if( mdir == Down )
				{
					direction = "down";
				}
				else if( mdir == Left )
				{
					direction = "left";
				}
				else if( mdir == Right )
				{
					direction = "right";
				}

				controls_config.SetParam( ctrl_id + ".mouse_type", string("motion") );
				controls_config.SetParam( ctrl_id + ".mouse_motion", direction );
			}
		}
		i++;
	}
	
	controls_config.Write( true, fname );
}

void GAMECONTROLS::LoadCalibration()
{
	ifstream cf;
	cf.open((game.settings.GetSettingsDir() + "/calibration").c_str());
	if (cf)
	{
		int i, m;
		for (i = 0; i < MAX_JOYSTICKS; i++)
		{
			for (m = 0; m < MAX_AXES; m++)
			{
				calibration[i][m][0] = game.utility.fGetParam(cf);
				calibration[i][m][1] = game.utility.fGetParam(cf);
				
				if (calibration[i][m][0] > -0.1)
					calibration[i][m][0] = -0.1;
				if (calibration[i][m][1] < 0.1)
					calibration[i][m][1] = 0.1;
			}
		}
	
		cf.close();
	}
}

void GAMECONTROLS::WriteCalibration()
{
	ofstream cf;
	cf.open((game.settings.GetSettingsDir() + "/calibration").c_str());
	if (cf)
	{
		cf << "# VDrift joystick calibration file.\n#Each line is an axis with the format [min] [max]" << endl << endl;
		int i, m;
		for (i = 0; i < MAX_JOYSTICKS; i++)
		{
			for (m = 0; m < MAX_AXES; m++)
				cf << calibration[i][m][0] << " " << calibration[i][m][1] << endl;
		}
		
		cf.close();
	}
	else
		cout << "Error writing calibration file to " + game.settings.GetSettingsDir() + "/calibration" << endl;
}

float GAMECONTROLS::GetCalibration(int joynum, int joyaxis, bool max)
{
	float c;
	
	if (joynum >= MAX_JOYSTICKS || joyaxis >= MAX_AXES)
	{
		if (max)
			return 1;
		else
			return -1;
	}

	if (max)
		c = calibration[joynum][joyaxis][1];
	else
		c = calibration[joynum][joyaxis][0];
	
	return c;
}

void GAMECONTROLS::InitJoy()
{
	int j;
	for (j = 0; j < SDL_NumJoysticks(); j++)
	{
		js[j] = SDL_JoystickOpen(j);
	}
}

void GAMECONTROLS::DeinitJoy()
{
	int j;
	for (j = 0; j < SDL_NumJoysticks(); j++)
		SDL_JoystickClose(js[j]);
}

void GAMECONTROLS::MinimizeCalibration()
{
	int i;
	for (i = 0; i < MAX_JOYSTICKS; i++)
	{
		int m;
		for (m = 0; m < MAX_AXES; m++)
		{
			calibration[i][m][0] = -0.1;
			calibration[i][m][1] = 0.1;
		}
	}
}

void GAMECONTROLS::SetCalibrationPoint(int joynum, int joyaxis, float val)
{
	if (val < calibration[joynum][joyaxis][0])
		calibration[joynum][joyaxis][0] = val;
	if (val > calibration[joynum][joyaxis][1])
		calibration[joynum][joyaxis][1] = val;
}

void GAMECONTROLS::UpdateSettings()
{
	bool got_type = game.settings.Get( "joystick.type", joysticktype );
	bool got_comp = game.settings.Get( "joystick.touchcomp", compensation );

	if( !got_type || ( joysticktype != "wheel" && joysticktype != "joystick" ) )
	{
		cout << "controls: invalid joystick type, default to joystick." << endl;
		joysticktype = "joystick";
	}

	if( !got_comp || ( compensation != "off" && compensation != "low" && compensation != "med" && compensation != "high" && compensation != "900to200" && compensation != "speed1" && compensation != "speed2") )
	{
		cout << "controls: invalid joystick touchiness compensation, default to off." << endl;
		compensation = "off";
	}
}

void GAMECONTROLS::DeleteControlsWithAction(string actionstr)
{
	list <CONTROL>::iterator i;
	bool found;
	bool done = false;
	
	while( !done )
	{
		found = false;
		for( i = controls.begin(); i != controls.end(); ++i )
		{
			if( i->GetName() == actionstr )
			{
				found = true;
				break;
			}
		}

		if( found )
		{
			controls.erase( i );
		}
		else if( i == controls.end() )
		{
			done = true;
		}
	}
}

void GAMECONTROLS::AddControl(CONTROL & other)
{
	controls.push_back(other);
}

float GAMECONTROLS::ApplyCalibration(int joynum, int joyaxis, float val)
{
	float max = GetCalibration(joynum,joyaxis,true);
	float min = GetCalibration(joynum,joyaxis,false);
	float range = max - min;
	float calgain = 2.0 / range;
	float offset = 1.0 - max * calgain;
	val = val * calgain + offset;
	
	//clip
	if (val < -1)
		val = -1;
	if (val > 1)
		val = 1;
	
	return val;
}

float GAMECONTROLS::ApplyDeadzone(float dz, float val)
{
	if (game.utility.dabs(val) < dz)
		val = 0;
	else
	{
		if (val < 0)
			val = (val + dz)*(1.0/(1.0-dz));
		else
			val = (val - dz)*(1.0/(1.0-dz));
	}
	
	return val;
}

float GAMECONTROLS::ApplyGain(float gain, float val)
{
	val *= gain;
	if (val < -1.0)
		val = -1.0;
	if (val > 1.0)
		val = 1.0;
	
	return val;
}

float GAMECONTROLS::ApplyExponent(float exponent, float val)
{
	val = pow(val, exponent);
	if (val < -1.0)
		val = -1.0;
	if (val > 1.0)
		val = 1.0;
	
	return val;
}

