/***************************************************************************
 *            textures.cpp
 *
 *  Thu Dec  1 17:46:50 2005
 *  Copyright  2005  Joe Venzon
 *  joe@venzon.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "textures.h"
#include "singleton.h"
#include "game.h"
extern GAME game;
#include "common.h"

//#define DEBUG_TEXTURES

void TEXTURE_SDLGL::LoadCubeVerticalCross()
{
	if (texture_info.GetName().empty())
		throw EXCEPTION(__FILE__, __LINE__, "Tried to load a texture with an empty name");
	
	if (loaded)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to double load texture");
	
	if (!texture_info.GetCube())
	{
		Load();
		return;
	}
	
	if (!texture_info.GetVerticalCross())
	{
		LoadCube();
		return;
	}
	
	//string cubefile;
	//cubefile = game.pathmanager.GetTexturePathFromName(texture_info.GetName());
	string cubefile = game.settings.GetDataDir() + "/" + texture_info.GetName();
	if (game.utility.FileExists(texture_info.GetName()))
		cubefile = texture_info.GetName();
	
	//glEnable(GL_TEXTURE_CUBE_MAP_ARB);
	
	GLuint new_handle = 0;
	glGenTextures(1, &new_handle);
	tex_id = new_handle;
	
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, new_handle);
	
	SDL_Surface * texture_surface = NULL;
	
	if ((texture_surface = IMG_Load(cubefile.c_str())))
	{
		for (int i = 0; i < 6; i++)
		{
			//bool error = false;
		
			//store dimensions
			/*if (i != 0 && (w != texture_surface->w || h != texture_surface->h))
			{
				throw EXCEPTION(__FILE__, __LINE__, "Cube map sides aren't equal sizes");
			}*/
			w = texture_surface->w/3;
			h = texture_surface->h/4;
			
			//detect channels
			int format = GL_RGB;
			switch (texture_surface->format->BytesPerPixel)
			{
				case 1:
					format = GL_LUMINANCE;
					break;
				case 2:
					format = GL_LUMINANCE_ALPHA;
					break;
				case 3:
					format = GL_RGB;
					break;
				case 4:
					format = GL_RGBA;
					break;
				default:
					throw EXCEPTION(__FILE__, __LINE__, "Texture has unknown format: " + texture_info.GetName() + " (" + cubefile + ")");
					break;
			}
			
			if (format != GL_RGB)
			{
				//throw EXCEPTION(__FILE__, __LINE__, "Cube map texture format isn't GL_RGB (this causes problems for some reason): " + texture_info.GetName() + " (" + cubefile + ")");
				//game.WriteDebuggingData("Warning:  Cube map texture format isn't GL_RGB (this causes problems for some reason): " + texture_info.GetName() + " (" + cubefile + ")");
			}
			
			int offsetx = 0;
			int offsety = 0;
			
			GLenum targetparam;
			if (i == 0)
			{
				targetparam = GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB;
				offsetx = 0;
				offsety = h;
			}
			else if (i == 1)
			{
				targetparam = GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB;
				offsetx = w*2;
				offsety = h;
			}
			else if (i == 2)
			{
				targetparam = GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB;
				offsetx = w;
				offsety = h*2;
			}
			else if (i == 3)
			{
				targetparam = GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB;
				offsetx = w;
				offsety = 0;
			}
			else if (i == 4)
			{
				targetparam = GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB;
				offsetx = w;
				offsety = h*3;
			}
			else if (i == 5)
			{
				targetparam = GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB;
				offsetx = w;
				offsety = h;
			}
			else
				throw EXCEPTION(__FILE__, __LINE__, "Iterated too far: " + texture_info.GetName() + " (" + cubefile + ")");
			
			/*offsetx = 0;
			offsety = 0;*/
			
			unsigned char cubeface[w*h*texture_surface->format->BytesPerPixel];
			
			if (i == 4) //special case for negative z
			{
				for (int yi = 0; yi < h; yi++)
				{
					for (int xi = 0; xi < w; xi++)
					{
						for (int ci = 0; ci < texture_surface->format->BytesPerPixel; ci++)
						{
							int idx1 = ((h-yi-1)+offsety)*texture_surface->w*texture_surface->format->BytesPerPixel + (w-xi-1+offsetx)*texture_surface->format->BytesPerPixel + ci;
							int idx2 = yi*w*texture_surface->format->BytesPerPixel+xi*texture_surface->format->BytesPerPixel+ci;
							cubeface[idx2] = ((unsigned char *)(texture_surface->pixels))[idx1];
							//cout << idx1 << "," << idx2 << endl;
						}
					}
				}
			}
			else
			{
				for (int yi = 0; yi < h; yi++)
				{
					for (int xi = 0; xi < w; xi++)
					{
						for (int ci = 0; ci < texture_surface->format->BytesPerPixel; ci++)
						{
							int idx1 = (yi+offsety)*texture_surface->w*texture_surface->format->BytesPerPixel+(xi+offsetx)*texture_surface->format->BytesPerPixel+ci;
							int idx2 = yi*w*texture_surface->format->BytesPerPixel+xi*texture_surface->format->BytesPerPixel+ci;
							cubeface[idx2] = ((unsigned char *)(texture_surface->pixels))[idx1];
							//cout << idx1 << "," << idx2 << endl;
						}
					}
				}
			}
			
			/*cubefiles[0] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-xp.png");
			cubefiles[1] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-xn.png");
			cubefiles[2] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-yn.png");
			cubefiles[3] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-yp.png");
			cubefiles[4] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-zn.png");
			cubefiles[5] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-zp.png");*/
			
			//glTexImage2D( targetparam, 0, format,texture_surface->w, texture_surface->h, 0, format, GL_UNSIGNED_BYTE, texture_surface->pixels );
			glTexImage2D( targetparam, 0, format, w, h, 0, format, GL_UNSIGNED_BYTE, cubeface );
		}
	}
	else
	{
		throw EXCEPTION(__FILE__, __LINE__, "Error loading texture file: " + texture_info.GetName() + " (" + cubefile + ")");
	}

	if (texture_surface)
	{
		// Free up any memory we may have used
		SDL_FreeSurface( texture_surface );
		texture_surface = NULL;
	}
	
	//glTexParameteri (GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_REPEAT);
	//glTexParameteri (GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	
	loaded = true;
	glDisable(GL_TEXTURE_CUBE_MAP_ARB);
}

void TEXTURE_SDLGL::LoadCube()
{
	if (texture_info.GetName().empty())
		throw EXCEPTION(__FILE__, __LINE__, "Tried to load a texture with an empty name");
	
	if (loaded)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to double load texture");
	
	if (!texture_info.GetCube())
	{
		Load();
		return;
	}
	
	if (texture_info.GetVerticalCross())
	{
		LoadCubeVerticalCross();
		return;
	}
	
	string cubefiles[6];
	/*cubefiles[0] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-xp.png");
	cubefiles[1] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-xn.png");
	cubefiles[2] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-yn.png");
	cubefiles[3] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-yp.png");
	cubefiles[4] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-zn.png");
	cubefiles[5] = game.pathmanager.GetTexturePathFromName(texture_info.GetName()+"-zp.png");*/
	
	cubefiles[0] = texture_info.GetName()+"-xp.png";
	cubefiles[1] = (texture_info.GetName()+"-xn.png");
	cubefiles[2] = (texture_info.GetName()+"-yn.png");
	cubefiles[3] = (texture_info.GetName()+"-yp.png");
	cubefiles[4] = (texture_info.GetName()+"-zn.png");
	cubefiles[5] = (texture_info.GetName()+"-zp.png");
	
	//glEnable(GL_TEXTURE_CUBE_MAP_ARB);
	
	GLuint new_handle = 0;
	glGenTextures(1, &new_handle);
	tex_id = new_handle;
	
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, new_handle);
	
	for (int i = 0; i < 6; i++)
	{
		//bool error = false;
		
		SDL_Surface * texture_surface = NULL;
	
		if ((texture_surface = IMG_Load(cubefiles[i].c_str())))
		{
			//store dimensions
			if (i != 0 && (w != texture_surface->w || h != texture_surface->h))
			{
				throw EXCEPTION(__FILE__, __LINE__, "Cube map sides aren't equal sizes");
			}
			w = texture_surface->w;
			h = texture_surface->h;
			
			//detect channels
			int format = GL_RGB;
			switch (texture_surface->format->BytesPerPixel)
			{
				case 1:
					format = GL_LUMINANCE;
					break;
				case 2:
					format = GL_LUMINANCE_ALPHA;
					break;
				case 3:
					format = GL_RGB;
					break;
				case 4:
					format = GL_RGBA;
					break;
				default:
					throw EXCEPTION(__FILE__, __LINE__, "Texture has unknown format: " + texture_info.GetName() + " (" + cubefiles[i] + ")");
					break;
			}
			
			if (format != GL_RGB)
			{
				throw EXCEPTION(__FILE__, __LINE__, "Cube map texture format isn't GL_RGB (this causes problems for some reason): " + texture_info.GetName() + " (" + cubefiles[i] + ")");
			}
			
			// Create MipMapped Texture
			
			GLenum targetparam;
			if (i == 0)
				targetparam = GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB;
			else if (i == 1)
				targetparam = GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB;
			else if (i == 2)
				targetparam = GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB;
			else if (i == 3)
				targetparam = GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB;
			else if (i == 4)
				targetparam = GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB;
			else if (i == 5)
				targetparam = GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB;
			else
				throw EXCEPTION(__FILE__, __LINE__, "Iterated too far: " + texture_info.GetName() + " (" + cubefiles[i] + ")");
			
			glTexImage2D( targetparam, 0, format,texture_surface->w, texture_surface->h, 0, format, GL_UNSIGNED_BYTE, texture_surface->pixels );
		}
		else
		{
			throw EXCEPTION(__FILE__, __LINE__, "Error loading texture file: " + texture_info.GetName() + " (" + cubefiles[i] + ")");
		}
	
		if (texture_surface)
		{
			// Free up any memory we may have used
			SDL_FreeSurface( texture_surface );
			texture_surface = NULL;
		}
	}
	
	//glTexParameteri (GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_REPEAT);
	//glTexParameteri (GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	
	loaded = true;
	glDisable(GL_TEXTURE_CUBE_MAP_ARB);
}

void TEXTURE_SDLGL::Load()
{
	if (texture_info.GetName().empty())
		throw EXCEPTION(__FILE__, __LINE__, "Tried to load a texture with an empty name");
	
	if (loaded)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to double load texture");
	
	if (texture_info.GetCube())
	{
		LoadCube();
		return;
	}
	
	//string filepath = game.pathmanager.GetTexturePathFromName(texture_info.GetName());
	string filepath = game.settings.GetDataDir() + "/" + texture_info.GetName();
	if (game.utility.FileExists(texture_info.GetName()))
		filepath = texture_info.GetName();
	
	//bool error = false;
	
	GLuint new_handle = 0;
	
	SDL_Surface * orig_surface(NULL);
	if (texture_info.GetSurface())
	{
		orig_surface = texture_info.GetSurface();
	}
	else
	{
		orig_surface = IMG_Load(filepath.c_str());
	}
	
	string texture_size;
	game.settings.Get( "display.texture_size", texture_size );
	
	SDL_Surface * texture_surface(orig_surface);
	
	if (orig_surface)
	{	
		//scale texture down if necessary
		if (texture_size == "medium" || texture_size == "small")
		{
			int maxsize = 256;
			float minscale = 0.5;
			if (texture_size == "small")
			{
				maxsize = 128;
				minscale = 0.25;
			}
			float scalew = 1.0;
			float scaleh = 1.0;
			float scale = 1.0;
		
			if (orig_surface->w > maxsize)
				scalew = (float)maxsize / orig_surface->w;
			if (orig_surface->h > maxsize)
				scaleh = (float)maxsize / orig_surface->h;
		
			if (scalew < scaleh)
				scale = scalew;
			else
				scale = scaleh;
		
			if (scale < minscale) scale = minscale;
		
			if (scale < 1.0)
			{
				//cerr << "endian bug!?: " + filename << endl;
				texture_surface = zoomSurface (orig_surface, scale, scale, SMOOTHING_ON);
			}
		}
	
		//store dimensions
		w = texture_surface->w;
		h = texture_surface->h;
		
		//detect channels
		int format = GL_RGB;
		switch (texture_surface->format->BytesPerPixel)
		{
			case 1:
				format = GL_LUMINANCE;
				alphachannel = false;
				break;
			case 2:
				format = GL_LUMINANCE_ALPHA;
				alphachannel = true;
				break;
			case 3:
				format = GL_RGB;
				alphachannel = false;
				break;
			case 4:
				format = GL_RGBA;
				alphachannel = true;
				break;
			default:
				break;
		}
		
		glGenTextures(1, &new_handle);					// Create Texture
		
		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, new_handle);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		
		if (texture_info.GetMipMap())
		{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
			gluBuild2DMipmaps( GL_TEXTURE_2D, format, texture_surface->w, texture_surface->h, format, GL_UNSIGNED_BYTE, texture_surface->pixels );
		}
		else
		{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
			glTexImage2D( GL_TEXTURE_2D, 0, format,texture_surface->w, texture_surface->h, 0, format, GL_UNSIGNED_BYTE, texture_surface->pixels );
		}
		
		//check for anisotropy
		int anisotropy = 0;
		game.settings.Get( "display.anisotropic", anisotropy );
		if (game.gfxcard.GetCapability(CARDINFOTYPE::ANISOTROPY) && anisotropy > 0)
		{
			//enable anisotropy based on level set in config file
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, (float)anisotropy);
		}
	}
	else
	{
		throw EXCEPTION(__FILE__, __LINE__, "Error loading texture file: " + texture_info.GetName() + " (" + filepath + ")");
	}
	
	//free the texture surface separately if it's a scaled copy of the original
	if (texture_surface != orig_surface && texture_surface)
	{
		SDL_FreeSurface( texture_surface );
	}
	
	//free the original surface if it's not a custom surface (used for the track map)
	if (!texture_info.GetSurface() && orig_surface)
	{
		SDL_FreeSurface( orig_surface );
	}
	
	loaded = true;
	tex_id = new_handle;
}

void TEXTURE_SDLGL::Activate(bool blur_override) const
{
	if (!loaded)
		throw EXCEPTION(__FILE__, __LINE__, "Tried to activate unloaded texture");
	
	/*if (game.graphics.BlurPass() && !blur_override)
		return;

	const bool normalmap = texture_info.NormalMap() && game.graphics.NormalMapping();
	if (normalmap)
	{
		game.graphics.BeginNormalMapBind();
	}*/
	
	/*if (!normalmap)
	{
		game.graphics.SetAlphaMasking(alphachannel);
	}*/
	
	if (texture_info.GetCube())
	{
		glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, tex_id);
	}
	else
	{
		glBindTexture(GL_TEXTURE_2D, tex_id);
	}
	
	//if (normalmap)	game.graphics.EndNormalMapBind();
}

void TEXTURE_SDLGL::Unload()
{
	if (loaded)
	{
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURE_SDLGL::Unload(): unloading " << GetTextureInfo().GetName() << endl;
		#endif
		glDeleteTextures(1,&tex_id);
	}
	loaded = false;
}

void TEXTURE_HANDLE::Activate(bool blur_override) const
{
	if (tex.PtrValid())
	{
		tex.GetPtr()->Activate(blur_override);
	}
	else
	{
		//throw is too strict?
		throw EXCEPTION(__FILE__, __LINE__, "Tried to activate NULL texture handle");
	}
}

const bool TEXTURE_HANDLE::Load(const TEXTUREINFO & texinfo)
{
	if (tex.PtrValid())
	{
		if (tex.GetPtr()->IsEqualTo(texinfo))
		{
			#ifdef DEBUG_TEXTURES
			cout << "TEXTURE_HANDLE::Load(" << texinfo.GetName() << "): already loaded" << endl;
			#endif
			return true;
		}
		else
		{
			#ifdef DEBUG_TEXTURES
			cout << "TEXTURE_HANDLE::Load(" << texinfo.GetName() << "): unloading previous texture: " << tex.GetPtr()->GetTextureInfo().GetName() << endl;
			#endif
			Unload();
		}
	}
	else
	{
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURE_HANDLE::Load(" << texinfo.GetName() << "): not yet loaded, calling LoadTexture" << endl;
		#endif
	}
	
	TEXTURE_SDLGL * newtexptr = game.textures.LoadTexture(texinfo);
	
	if (newtexptr != NULL)
	{
		tex.SetPtr(*newtexptr);
		tex.GetPtr()->IncrementReferences();
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURE_HANDLE::Load(" << texinfo.GetName() << "): loaded " << tex.GetPtr() << ", references now " << tex.GetPtr()->NumReferences() << endl;
		#endif
	}
	else
	{
		throw EXCEPTION(__FILE__, __LINE__, "LoadTexture returned NULL pointer for "+texinfo.GetName());
		return false;
	}
	
	return true;
}

void TEXTURE_HANDLE::Unload()
{
	if (tex.PtrValid())
	{
		tex.GetPtr()->DecrementReferences();
		
		if (tex.GetPtr()->NumReferences() <= 0)
		{
			game.textures.UnloadTexture(*tex.GetPtr());
		}
	}
	else
	{
		#ifdef DEBUG_TEXTURES
		cout << "3TEXTURE_HANDLE::Unload(): already unloaded, twiddling thumbs" << endl;
		#endif
	}

	tex.Invalidate();
}

void TEXTURE_SDLGL::DecrementReferences()
{
	references--;
	
	if (references <= 0)
	{
		#ifdef DEBUG_TEXTURES
		cout << "1TEXTURE_SDLGL::DecrementReferences(" << texture_info.GetName() << "): references <= 0, unloading now" << endl;
		#endif
		
		if (references < 0)
		{
			stringstream err;
			err << "texture " + texture_info.GetName() + " has < 0 references, now " << references;
			game.WriteDebuggingData(err.str()); //do this to ensure the message gets written since this function may be running outside of a catch (called by a destructor)
			throw EXCEPTION(__FILE__, __LINE__, err.str());
		}
	}
	else
	{
		#ifdef DEBUG_TEXTURES
		cout << "2TEXTURE_SDLGL::DecrementReferences(" << texture_info.GetName() << "): references--, now " << NumReferences() << endl;
		#endif
	}
}

void TEXTURES::ReloadAll()
{
	ITERATE(TEXTURE_SDLGL, textures, tex)
	{
		tex->Unload();
		tex->Load();
	}
}

TEXTURE_SDLGL * TEXTURES::LoadTexture( const TEXTUREINFO & texinfo, int &w, int &h )
{
	TEXTURE_SDLGL * texptr = FindTexture(texinfo);

	if (texptr == NULL)
	{
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURES::LoadTexture(" << texinfo.GetName() << "): not yet loaded, loading now" << endl;
		#endif
		
		//texptr = textures.DB_Add();
		TEXTURE_SDLGL newtex;
		textures.push_back(newtex);
		TEXTURE_SDLGL & newtexref = textures.back();
		texptr = &(textures.back());
		//cout << "texture list size: " << textures.size() << endl;
		
		//actually load the texture
		newtexref.SetInfo(texinfo);
		newtexref.Load();
		w = newtexref.GetW();
		h = newtexref.GetH();
	}
	else
	{
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURES::LoadTexture(" << texinfo.GetName() << "): already loaded" << endl;
		#endif
	}
	
	return texptr;
}

void TEXTURES::UnloadTexture(const TEXTURE_SDLGL & textodel)
{
	//for (list <TEXTURE>::iterator i = textures.begin(); i != textures.end(); i++)
	list < list <TEXTURE_SDLGL>::iterator > deletelist;
	ITERATE(TEXTURE_SDLGL, textures, i)
	{
		if (i->IsEqualTo(textodel))
		//if (&(*i) == &textodel)
		{
			if (i->NumReferences() > 0)
			{
				stringstream err;
				err << "Got asked to delete a texture with > 0 references: " + i->GetTextureInfo().GetName() << " (" << textodel.GetTextureInfo().GetName() << ")";
				game.WriteDebuggingData(err.str()); //do this to ensure the message gets written since this function may be running outside of a catch (called by a destructor)
				throw EXCEPTION(__FILE__, __LINE__, err.str());
			}
			#ifdef DEBUG_TEXTURES
			cout << "Going to delete texture " << i->GetTextureInfo().GetName() << endl;
			#endif
			deletelist.push_back(i);
		}
	}
	if (deletelist.empty())
	{
		stringstream err;
		err << "Couldn't find texture to delete: " + textodel.GetTextureInfo().GetName();
		game.WriteDebuggingData(err.str()); //do this to ensure the message gets written since this function may be running outside of a catch (called by a destructor)
		throw EXCEPTION(__FILE__, __LINE__, err.str());
	}
	ITERATE(list <TEXTURE_SDLGL>::iterator, deletelist, i)
	{
		textures.erase(*i);
	}
}

TEXTURE_SDLGL * TEXTURES::FindTexture(const TEXTUREINFO & texinfo)
{
	TEXTURE_SDLGL * tptr = NULL;
	
	ITERATE(TEXTURE_SDLGL, textures, i)
	{
		if (i->IsEqualTo(texinfo))
			tptr = &(*i);
	}

	return tptr;
}

TEXTURE_HANDLE & TEXTURE_HANDLE::CopyFrom(const TEXTURE_HANDLE & other)
{
	//Unload();
	
	//tex = other.tex;
	//tex->references++;
	
	if (!other.Loaded())
	{
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURE_HANDLE::CopyFrom(): other texture handle is NULL" << endl;
		#endif
		Unload();
		return *this;
	}
	else
	{
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURE_HANDLE::CopyFrom() called" << endl;
		cout << "TEXTURE_HANDLE::CopyFrom(" + other.GetTexture().GetTextureInfo().GetName() + "): calling Load(" << other.GetTexture().GetTextureInfo().GetName() << ")" << endl;
		#endif
		Load(other.GetTexture().GetTextureInfo());
		#ifdef DEBUG_TEXTURES
		cout << "TEXTURE_HANDLE::CopyFrom(" + tex.GetPtr()->GetTextureInfo().GetName() + "): loaded, references now " << tex.GetPtr()->NumReferences() << endl;
		#endif
	}
	
	return *this;
}

TEXTURE_HANDLE::TEXTURE_HANDLE(const TEXTURE_HANDLE & other)
{
	//tex = NULL;
	#ifdef DEBUG_TEXTURES
	cout << "TEXTURE_HANDLE Copy constructor called" << endl;
	#endif
	CopyFrom(other);
}

TEXTURE_HANDLE::~TEXTURE_HANDLE()
{
	Unload();
}

const bool TEXTURE_SDLGL::IsEqualTo(const TEXTUREINFO & texinfo) const
{
	#ifdef DEBUG_TEXTURES
	cout << "TEXTURE_SDLGL::IsEqualTo(name test): " << texinfo.GetName() << "?=?" << texture_info.GetName();
	//cout << "TEXTURE_SDLGL::IsEqualTo(mipmap test): " << texinfo.GetMipMap() << "?=?" << texture_info.GetMipMap() << endl;
	cout << " Going to return " << (texinfo.GetName() == texture_info.GetName() && texinfo.GetMipMap() == texture_info.GetMipMap()) << endl;
	#endif
	return (texinfo.GetName() == texture_info.GetName() && texinfo.GetMipMap() == texture_info.GetMipMap());
}

const bool TEXTURE_HANDLE::Load( string new_tex_file, SDL_Surface* surface, bool mipmap, bool cube, int &w, int &h )
{
	TEXTUREINFO newinfo;
	newinfo.SetName(new_tex_file);
	newinfo.SetSurface(surface);
	newinfo.SetMipMap(mipmap);
	newinfo.SetCube(cube, cube);
	
	bool success = Load(newinfo);
	if (success)
	{
		w = tex.GetRef().GetW();
		h = tex.GetRef().GetH();
		return true;
	}
	else
		return false;
}
